@@ -4,9 +4,9 @@ import ctypes as ct
 import os
 import sys
 import weakref
-import version
 
-from util import find_library
+from . import version
+from .util import find_library
 
 XT_INV_PROTO = 0x40  # invert the sense of PROTO
 
@@ -1036,96 +1036,3 @@ class xtables(object):
         flags = ct.pointer(ct.c_uint(0))
         self._parse(m, argv, invert, flags, fw, ptr)
         m.mflags |= flags[0]
-
-    # Check that all option constraints have been met. This effectively
-    # replaces ->final_check of the older API.
-    def _options_fcheck(self, name, xflags, table):
-        for entry in table:
-            if entry.name is None:
-                break
-            if entry.flags & XTOPT_MAND and not xflags & (1 << entry.id):
-                raise XTablesError("%s: --%s must be specified" % (name,
-                                                                   entry.name))
-                if not xflags & (1 << entry.id):
-                    continue
-            # XXX: check for conflicting options
-
-    def _fcheck_target_old(self, target):
-        # old API
-        if not target.final_check:
-            return
-        rv = _wrap_uintfn(target.final_check, target.tflags)
-        if rv:
-            raise XTablesError("%s.final_check() has failed" %
-                               (target.name))
-
-    def _fcheck_target_new(self, target):
-        # new API
-        cb = xt_fcheck_call()
-        cb.ext_name = target.name
-        cb.data = ct.cast(target.t[0].data, ct.c_void_p)
-        cb.xflags = target.tflags
-        cb.udata = target.udata
-        rv = _wrap_x6fn(target.x6_fcheck, ct.pointer(cb))
-        if rv:
-            raise XTablesError("%s.x6_fcheck has failed" % (target.name))
-        if target.x6_options:
-            self._options_fcheck(target.name, target.tflags,
-                                 target.x6_options)
-
-    # Dispatch arguments to the appropriate final_check function, based upon
-    # the extension's choice of API.
-    @preserve_globals
-    def final_check_target(self, target):
-        x6_fcheck = None
-        try:
-            # new API?
-            x6_fcheck = target.x6_fcheck
-        except AttributeError:
-            # old API
-            pass
-
-        if x6_fcheck:
-            self._fcheck_target_new(target)
-        else:
-            self._fcheck_target_old(target)
-
-    def _fcheck_match_old(self, match):
-        # old API
-        if not match.final_check:
-            return
-        rv = _wrap_uintfn(match.final_check, match.mflags)
-        if rv:
-            raise XTablesError("%s.final_check() has failed" %
-                               (match.name))
-
-    def _fcheck_match_new(self, match):
-        # new API
-        cb = xt_fcheck_call()
-        cb.ext_name = match.name
-        cb.data = ct.cast(match.m[0].data, ct.c_void_p)
-        cb.xflags = match.mflags
-        cb.udata = match.udata
-        rv = _wrap_x6fn(match.x6_fcheck, ct.pointer(cb))
-        if rv:
-            raise XTablesError("%s.x6_fcheck has failed" % (match.name))
-        if match.x6_options:
-            self._options_fcheck(match.name, match.mflags,
-                                 match.x6_options)
-
-    # Dispatch arguments to the appropriate final_check function, based upon
-    # the extension's choice of API.
-    @preserve_globals
-    def final_check_match(self, match):
-        x6_fcheck = None
-        try:
-            # new API?
-            x6_fcheck = match.x6_fcheck
-        except AttributeError:
-            # old API
-            pass
-
-        if x6_fcheck:
-            self._fcheck_match_new(match)
-        else:
-            self._fcheck_match_old(match)
